/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* global window*/

/**
 * Application common utility module.
 * Provides common methods used by other modules.
 *
 * @module app.common
 * @namespace app.common
 * @memberof app
 */

// make sure that "app" namespace is created
window.app = window.app || {};

// strict mode wrapper function
(function defineAppCommon(app) {
    'use strict';

    /**
     * Returns string with escaped HTML special characters.
     *
     * @memberof app.common
     * @public
     * @param {string} string
     * @returns {string}
     */
    function escapeHTML(string) {
        return string
            .replace(/&/g, '&amp;')
            .replace(/</g, '&lt;')
            .replace(/>/g, '&gt;')
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&#039;');
    }

    /**
     * Returns true if specified number contains integer type value,
     * false otherwise.
     *
     * @memberof app.common
     * @public
     * @param {number} number
     * @returns {boolean}
     */
    function isInteger(number) {
        return number % 1 === 0;
    }

    /**
     * Returns closest HTML element (including itself) satisfying
     * specified selector.
     * If element is not found, null value is returned.
     *
     * @memberof app.common
     * @public
     * @param {HTMLElement} element
     * @param {string} selector
     * @returns {HTMLElement}
     */
    function closestElement(element, selector) {
        do {
            if (element.webkitMatchesSelector(selector)) {
                return element;
            }

            element = element.parentElement;

        } while (element !== null);

        return null;
    }

    app.common = {
        escapeHTML: escapeHTML,
        isInteger: isInteger,
        closestElement: closestElement
    };

})(window.app);
