/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, tizen, console */

/**
 * The application model module.
 * Provides access to application preferences.
 *
 * @module app.model
 * @namespace app.model
 * @memberof app
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppModel(app) {
    'use strict';

    /**
     * Returns (via callback) all preferences data.
     *
     * @memberof app.model
     * @public
     * @param {function} successCallback
     * @param {function} errorCallback
     */
    function getPreferences(successCallback, errorCallback) {
        try {
            tizen.preference.getAll(successCallback, errorCallback);
        } catch (error) {
            errorCallback(error);
        }
    }

    /**
     * Returns a preference value for a given key.
     *
     * In case of any error or when preference with given key does not exist,
     * null value is returned.
     *
     * @memberof app.model
     * @public
     * @param {string} key
     * @returns {PreferenceValueType|null}
     */
    function getPreferenceValue(key) {
        try {
            return tizen.preference.getValue(key);
        } catch (error) {
            console.error('Unable to get preference value', error);
            return null;
        }
    }

    /**
     * Sets the preference value.
     * Returns true if value was successfully set, false otherwise.
     *
     * @memberof app.model
     * @public
     * @param {string} key
     * @param {PreferenceValueType} value
     * @returns {boolean}
     */
    function setPreferenceValue(key, value) {
        try {
            tizen.preference.setValue(key, value);
            return true;
        } catch (error) {
            console.error('Unable to set preference value', error);
            return false;
        }
    }

    /**
     * Removes all key-value pairs from the preferences.
     * Returns true if operation ends successfully, false in case of any error.
     *
     * @memberof app.model
     * @public
     * @returns {boolean}
     */
    function removePreferences() {
        try {
            tizen.preference.removeAll();
            return true;
        } catch (error) {
            console.error('Unable to remove preferences', error);
            return false;
        }
    }

    /**
     * Removes preference with specified key.
     * Returns true if operation ends successfully, false in case of any error.
     *
     * @memberof app.model
     * @public
     * @param {string} key Preference key.
     * @returns {boolean}
     */
    function removePreference(key) {
        try {
            tizen.preference.remove(key);
            return true;
        } catch (error) {
            console.error('Unable to remove preference', error);
            return false;
        }
    }

    app.model = {
        getPreferences: getPreferences,
        getPreferenceValue: getPreferenceValue,
        setPreferenceValue: setPreferenceValue,
        removePreference: removePreference,
        removePreferences: removePreferences
    };

})(window.app);
